package cs2.introml.tests;

import cs2.introml.DenseExample;
import cs2.introml.Example;
import cs2.introml.FeatureEntry;
import cs2.introml.SparseExample;
import junit.framework.TestCase;

public class P4Tests extends TestCase {
	
	public void testString() {
		
		final String classification = "string";
		final double[] features = new double[] { 3., 1., 4. };
		
		final DenseExample exD1 = new DenseExample( classification, features );
		assertTrue( exD1.toString().equals( classification + " 1:" + features[0] + " 2:" + features[1] + " 3:" + features[2] ) );
		
		final DenseExample exD2 = new DenseExample( classification, features.length );
		for ( int i=0; i<features.length; i++ ) {
			exD2.setFeature( i, features[i] );
		}
		assertTrue( exD2.toString().equals( classification + " 1:" + features[0] + " 2:" + features[1] + " 3:" + features[2] ) );
		
		final SparseExample exS = new SparseExample( classification, features.length );
		assertTrue( exS.toString().equals( classification ) );
		
		for ( int i=0; i<features.length; i++ ) {
			exS.setFeature( i, features[i] );
		}
		assertTrue( exS.toString().equals( classification + " 1:" + features[0] + " 2:" + features[1] + " 3:" + features[2] ) );
		
		exS.removeFeature( 0 );
		assertTrue( exS.toString().equals( classification + " 2:" + features[1] + " 3:" + features[2] ) );
		
		exS.setFeature( 2, 0.0 );
		assertTrue( exS.toString().equals( classification + " 2:" + features[1] ) );
		
		exS.setFeature( 1, 7. );
		assertTrue( exS.toString().equals( classification + " 2:" + 7. ) );
		
	}

	public void testDenseArray() {
		
		final String classification = "string";
		final double[] features = new double[] { 3., 1., 4., 1., 5., 9., Math.PI };
		
		final Example ex = new DenseExample( classification, features );
		int i;
		
		assertEquals( features.length, ex.getNumFeatures() );
		assertEquals( classification, ex.getResult() );
		
		i = 0;
		for ( FeatureEntry e : ex ) {
			assertTrue( e.getNumber() == i );
			assertTrue( e.getFeature() == features[ i ] );
			i++;
		}
		
		for ( i=0; i<ex.getNumFeatures(); i++ ) {
			assertEquals( features[i], ex.getFeature( i ) );
		}
		
	}
	
	public void testDenseIndividual() {
		
		final P4Tests classification = this;
		final double[] features = new double[] { 2., 7., 1., 8., Math.E };
		
		int i = 0, j = 0;
		final Example ex = new DenseExample( classification, features.length ).setFeature( i++, features[ j++ ] ).setFeature( i++, features[ j++ ] ).setFeature( i++, features[ j++ ] ).setFeature( i++, features[ j++ ] ).setFeature( i++, features[ j++ ] );
		
		assertEquals( features.length, ex.getNumFeatures() );
		assertEquals( classification, ex.getResult() );
		
		i = 0;
		for ( FeatureEntry e : ex ) {
			assertTrue( e.getNumber() == i );
			assertTrue( e.getFeature() == features[ i ] );
			i++;
		}
		
		for ( i=0; i<ex.getNumFeatures(); i++ ) {
			assertEquals( features[i], ex.getFeature( i ) );
		}
		
	}
	
	public void testSparse() {
		
		final String classification = "test";
		final int numFeatures = 20;
		
		final SparseExample ex = new SparseExample( classification, numFeatures );
		
		assertEquals( classification, ex.getResult() );
		assertEquals( numFeatures, ex.getNumFeatures() );
		
		for ( int i=0; i<numFeatures; i++ ) {
			assertEquals( 0., ex.getFeature( i ) );
		}
		assertNull( ex.getFeature( numFeatures+1 ) );
		
		for ( FeatureEntry e : ex ) {
			assertEquals( 0., e.getFeature() );
		}
		
		for ( FeatureEntry e : ex.sparseFeatures() ) {
			assertNull( e );
		}
		
		// needs to be at least 2 long, ascending order
		final int[] keys = { 2, 3, 5, 7, 11, 13, 17 };
		final double[] values = { 1, 2, 3, 4, 5, 6, 7 };
		assertEquals( keys.length, values.length );
		
		ex.setFeature( keys[0], values[0] ).setFeature( keys[1], values[1] );
		
		int i;
		for ( i=2; i<keys.length; i++ ) {
			ex.setFeature( keys[i], values[i] );
		}
		
		i = 0;
		for ( FeatureEntry e : ex.sparseFeatures() ) {
			assertTrue( e.getNumber() == keys[i] );
			assertTrue( e.getFeature() == values[ i ] );
			i++;
		}
		
		int j = 0;
		for ( i=0; i<ex.getNumFeatures(); i++ ) {
			
			if ( j < keys.length ) {
				if ( i == keys[j] ) {
					assertEquals( values[j], ex.getFeature( i ) );
					j++;
				} else {
					assertEquals( 0., ex.getFeature( i ) );
				}
			} else {
				assertEquals( 0., ex.getFeature( i ) );
			}
			
		}
		
		j = 0;
		for ( FeatureEntry e : ex ) {
			
			if ( j < keys.length ) {
				if ( e.getNumber() == keys[j] ) {
					assertEquals( values[j], e.getFeature() );
					j++;
				} else {
					assertEquals( 0., e.getFeature() );
				}
			} else {
				assertEquals( 0., e.getFeature() );
			}
			
		}
		
	}

}
